@@ -29,7 +29,6 @@
 #include <boost/serialization/export.hpp>
 
 const ButtonInfo default_button(Button2);
-const int default_pressure_threshold = 192;
 
 PrefDB::PrefDB() :
 	TimeoutWatcher(5000),
@@ -37,8 +36,6 @@ PrefDB::PrefDB() :
 	button(default_button),
 	trace(TraceDefault),
 	advanced_ignore(false),
-	pressure_abort(false),
-	pressure_threshold(default_pressure_threshold),
 	proximity(false),
 	feedback(true),
 	left_handed(false),
@@ -46,7 +43,7 @@ PrefDB::PrefDB() :
 	final_timeout(250),
 	timeout_profile(TimeoutDefault),
 	timeout_gestures(false),
-	tray_icon(true),
+	tray_icon(false),
 	color(Gdk::Color("#980101")),
 	trace_width(3),
 	advanced_popups(true),
@@ -54,7 +51,8 @@ PrefDB::PrefDB() :
 	scroll_speed(2.0),
 	tray_feedback(false),
 	show_osd(true),
-	move_back(false)
+	move_back(false),
+	whitelist(false)
 {}
 
 template<class Archive> void PrefDB::serialize(Archive & ar, const unsigned int version) {
@@ -98,8 +96,10 @@ template<class Archive> void PrefDB::serialize(Archive & ar, const unsigned int
 	ar & timing_workaround;
 	bool show_clicks = false;
 	ar & show_clicks;
-	ar & pressure_abort.unsafe_ref();
-	ar & pressure_threshold.unsafe_ref();
+	bool pressure_abort = false;
+	ar & pressure_abort;
+	int pressure_threshold = 192;
+	ar & pressure_threshold;
 	ar & proximity.unsafe_ref();
 	if (version < 5) return;
 	ar & feedback.unsafe_ref();
@@ -133,6 +133,8 @@ template<class Archive> void PrefDB::serialize(Archive & ar, const unsigned int
 	ar & move_back.unsafe_ref();
 	if (version < 17) return;
 	ar & device_timeout.unsafe_ref();
+	if (version < 18) return;
+	ar & whitelist.unsafe_ref();
 }
 
 void PrefDB::timeout() {
