@@ -18,10 +18,14 @@
 #include "main.h"
 #include "grabber.h"
 #include <glibmm/i18n.h>
+#include <sys/stat.h>
+#include "cellrenderertextish.h"
 
 #include <set>
 #include <iostream>
 
+extern const char *desktop_file;
+
 class Check : private Base {
 	IO<bool> &io;
 	Gtk::CheckButton *check;
@@ -116,7 +120,7 @@ public:
 		combo = Gtk::manage(new Gtk::ComboBoxText);
 		parent->add(*combo);
 		for (const Info *i = info; i->name; i++)
-			combo->append_text(_(i->name));
+			combo->append(_(i->name));
 		notify();
 		combo->signal_changed().connect(sigc::mem_fun(*this, &Combo::on_changed));
 		combo->show();
@@ -183,9 +187,6 @@ const Combo<TimeoutType>::Info timeout_info_exp[] = {
 
 Source<bool> autostart_ok(true);
 
-#include <sys/stat.h>
-extern const char *desktop_file;
-
 class Autostart : public IO<bool>, private Base {
 	bool a;
 	std::string filename;
@@ -208,8 +209,11 @@ public:
 	virtual bool get() const { return a; }
 	virtual void notify() {
 		if (a) {
+			char path[256] = "easystroke";
+			readlink("/proc/self/exe", path, sizeof(path));
+
 			FILE *file = fopen(filename.c_str(), "w");
-			if (!file || fprintf(file, desktop_file, "easystroke") == -1)
+			if (!file || fprintf(file, desktop_file, path) == -1)
 				autostart_ok.set(false);
 			if (file)
 				fclose(file);
@@ -231,6 +235,10 @@ void remove_last_entry(const Glib::ustring & name) {
 	combo_model->erase(--i);
 }
 
+static void on_prefs_editing_started(GtkCellRenderer *, GtkCellEditable *editable, const gchar *path, gpointer data) {
+	((Prefs *)data)->on_button_editing_started(editable, path);
+}
+
 Prefs::Prefs() {
 	new Check(prefs.advanced_ignore, "check_advanced_ignore");
 
@@ -278,6 +286,7 @@ Prefs::Prefs() {
 	widgets->get_widget("treeview_exceptions", tv);
 	widgets->get_widget("treeview_devices", dtv);
 	widgets->get_widget("treeview_extra", etv);
+	widgets->get_widget("frame_tablet", frame_tablet);
 
 	new Sensitive(*fun(&is_custom, prefs.timeout_profile), "hbox_timeout");
 	new Sensitive(*fun(&draw_trace, prefs.trace), "button_color");
@@ -288,13 +297,12 @@ Prefs::Prefs() {
 	tv->append_column(_("Application (WM__CLASS)"), cols.user_app);
 	tm->set_sort_column(cols.user_app, Gtk::SORT_ASCENDING);
 
-	CellRendererTextish *button_renderer = Gtk::manage(new CellRendererTextish);
-	button_renderer->mode = CellRendererTextish::POPUP;
-	tv->append_column(_("Button"), *button_renderer);
-	Gtk::TreeView::Column *col_button = tv->get_column(1);
-	col_button->add_attribute(button_renderer->property_text(), cols.button);
-	button_renderer->property_editable() = true;
-	button_renderer->signal_editing_started().connect(sigc::mem_fun(*this, &Prefs::on_button_editing_started));
+	CellRendererTextish *button_renderer = cell_renderer_textish_new ();
+	button_renderer->mode = CELL_RENDERER_TEXTISH_MODE_Popup;
+	GtkTreeViewColumn *col_button = gtk_tree_view_column_new_with_attributes(_("Button"), GTK_CELL_RENDERER (button_renderer), "text", cols.button.index(), NULL);
+	gtk_tree_view_append_column(tv->gobj(), col_button);
+	g_object_set(button_renderer, "editable", true, NULL);
+	g_signal_connect(button_renderer, "editing-started", G_CALLBACK(on_prefs_editing_started), this);
 
 	bbutton->signal_clicked().connect(sigc::mem_fun(*this, &Prefs::on_select_button));
 
@@ -345,9 +353,6 @@ Prefs::Prefs() {
 		widgets->get_widget("hbox_timeout", hbox);
 		hbox->hide();
 	}
-	Gtk::Frame *tablet_frame;
-	widgets->get_widget("frame_advanced2", tablet_frame);
-	tablet_frame->hide();
 
 	set_button_label();
 
@@ -361,10 +366,13 @@ Prefs::Prefs() {
 }
 
 void Prefs::update_device_list() {
+	bool proximity = false;
 	ignore_device_toggled = true;
 	dtm->clear();
 	std::set<std::string> names;
 	for (Grabber::DeviceMap::iterator i = grabber->xi_devs.begin(); i != grabber->xi_devs.end(); ++i) {
+		if (i->second->proximity_axis >= 0)
+			proximity = true;
 		std::string name = i->second->name;
 		if (names.count(name))
 			continue;
@@ -382,6 +390,7 @@ void Prefs::update_device_list() {
 					row[dcs.timeout] = i->name;
 	}
 	ignore_device_toggled = false;
+	frame_tablet->set_visible(proximity);
 }
 
 void Prefs::update_extra_buttons() {
@@ -468,7 +477,7 @@ SelectButton::SelectButton(ButtonInfo bi, bool def, bool any) {
 		select_button = Gtk::manage(new Gtk::ComboBoxText);
 		box_button->add(*select_button);
 		for (int i = 1; i <= 12; i++)
-			select_button->append_text(Glib::ustring::compose(_("Button %1"), i));
+			select_button->append(Glib::ustring::compose(_("Button %1"), i));
 		select_button->show();
 	}
 	select_button->set_active(bi.button-1);
@@ -606,7 +615,7 @@ bool Prefs::select_row(const Gtk::TreeModel::Path& path, const Gtk::TreeModel::i
 }
 
 void Prefs::on_add() {
-	std::string str = select_window();
+	std::string str = grabber->select_window();
 	bool is_new;
 	{
 		Atomic a;
@@ -625,7 +634,7 @@ void Prefs::on_add() {
 	}
 }
 
-void Prefs::on_button_editing_started(Gtk::CellEditable* editable, const Glib::ustring& path) {
+void Prefs::on_button_editing_started(GtkCellEditable* editable, const gchar *path) {
 	Gtk::TreeRow row(*tm->get_iter(path));
 	std::string app = row[cols.app];
 	ButtonInfo bi;
